import { HtmlSanitizer, matchLink, wrap } from 'roosterjs-editor-dom';
// Regex matching Uri scheme
var URI_REGEX = /^[a-zA-Z]+:/i;
// Regex matching begin of email address
var MAILTO_REGEX = /^[\w.%+-]+@/i;
// Regex matching begin of ftp, i.e. ftp.microsoft.com
var FTP_REGEX = /^ftp\./i;
function applyLinkPrefix(url) {
    if (!url) {
        return url;
    }
    // Add link prefix per rule:
    // (a) if the url always starts with a URI scheme, leave it as it is
    // (b) if the url is an email address, xxx@... add mailto: prefix
    // (c) if the url starts with ftp., add ftp:// prefix
    // (d) rest, add http:// prefix
    var prefix = '';
    if (url.search(URI_REGEX) < 0) {
        if (url.search(MAILTO_REGEX) == 0) {
            prefix = 'mailto:';
        }
        else if (url.search(FTP_REGEX) == 0) {
            prefix = 'ftp://';
        }
        else {
            // fallback to http://
            prefix = 'http://';
        }
    }
    return prefix + url;
}
/**
 * Insert a hyperlink at cursor.
 * When there is a selection, hyperlink will be applied to the selection,
 * otherwise a hyperlink will be inserted to the cursor position.
 * @param editor Editor object
 * @param link Link address, can be http(s), mailto, notes, file, unc, ftp, news, telnet, gopher, wais.
 * When protocol is not specified, a best matched protocol will be predicted.
 * @param altText Optional alt text of the link, will be shown when hover on the link
 * @param displayText Optional display text for the link.
 * @param target Optional display target for the link ("_blank"|"_self"|"_parent"|"_top"|"{framename}")
 * If specified, the display text of link will be replaced with this text.
 * If not specified and there wasn't a link, the link url will be used as display text.
 */
export default function createLink(editor, link, altText, displayText, target) {
    editor.focus();
    var url = (checkXss(link) || '').trim();
    if (url) {
        var linkData = matchLink(url);
        // matchLink can match most links, but not all, i.e. if you pass link a link as "abc", it won't match
        // we know in that case, users will want to insert a link like http://abc
        // so we have separate logic in applyLinkPrefix to add link prefix depending on the format of the link
        // i.e. if the link starts with something like abc@xxx, we will add mailto: prefix
        // if the link starts with ftp.xxx, we will add ftp:// link. For more, see applyLinkPrefix
        var normalizedUrl_1 = linkData ? linkData.normalizedUrl : applyLinkPrefix(url);
        var originalUrl_1 = linkData ? linkData.originalUrl : url;
        editor.addUndoSnapshot(function () {
            var selection = editor.getSelectionRangeEx();
            var anchor = null;
            if (selection.type === 0 /* Normal */) {
                var range = selection.ranges[0];
                if (range && range.collapsed) {
                    anchor = getAnchorNodeAtCursor(editor);
                    // If there is already a link, just change its href
                    if (anchor) {
                        anchor.href = normalizedUrl_1;
                        // Change text content if it is specified
                        updateAnchorDisplayText(anchor, displayText);
                    }
                    else {
                        anchor = editor.getDocument().createElement('A');
                        anchor.textContent = displayText || originalUrl_1;
                        anchor.href = normalizedUrl_1;
                        editor.insertNode(anchor);
                    }
                }
                else {
                    // the selection is not collapsed, use browser execCommand
                    editor
                        .getDocument()
                        .execCommand("createLink" /* CreateLink */, false, normalizedUrl_1);
                    var traverser = editor.getSelectionTraverser();
                    var currentInline = traverser === null || traverser === void 0 ? void 0 : traverser.getNextInlineElement();
                    // list for removing unwanted lines
                    var deletionInlineList = [];
                    while (currentInline) {
                        deletionInlineList.push(currentInline.getContainerNode());
                        currentInline = traverser === null || traverser === void 0 ? void 0 : traverser.getNextInlineElement();
                    }
                    deletionInlineList.forEach(function (node) { return editor.deleteNode(node); });
                    anchor = getAnchorNodeAtCursor(editor);
                    updateAnchorDisplayText(anchor, displayText);
                }
            }
            else if (selection.type === 2 /* ImageSelection */) {
                anchor = wrap(selection.image, 'A');
                anchor.href = normalizedUrl_1;
            }
            if (altText && anchor) {
                anchor.title = altText;
            }
            if (anchor) {
                updateAnchorTarget(anchor, target);
            }
            return anchor;
        }, "CreateLink" /* CreateLink */);
    }
}
function getAnchorNodeAtCursor(editor) {
    return editor.queryElements('a[href]', 1 /* OnSelection */)[0];
}
function updateAnchorDisplayText(anchor, displayText) {
    if (displayText && anchor.textContent != displayText) {
        anchor.textContent = displayText;
    }
}
function updateAnchorTarget(anchor, target) {
    if (target) {
        anchor.target = target;
    }
    else if (!target && anchor.getAttribute('target')) {
        anchor.removeAttribute('target');
    }
}
function checkXss(link) {
    var sanitizer = new HtmlSanitizer();
    var a = document.createElement('a');
    a.href = link || '';
    sanitizer.sanitize(a);
    // We use getAttribute because some browsers will try to make the href property a valid link.
    // This has unintended side effects when the link lacks a protocol.
    return a.getAttribute('href');
}
//# sourceMappingURL=createLink.js.map